/* RIGS.h - This file is included in all ObjC file of RIGS
    It contains all common definitions

   $Id: RIGS.h,v 1.1.1.1 2001/09/12 21:29:38 ljulliar Exp $

   Copyright (C) 2001 Free Software Foundation, Inc.

   Written by:  Laurent Julliard <laurent@julliard-online.org>
   Date: Aug 2001
   
   This file is part of the GNUstep Ruby  Interface Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.

*/

#ifndef __RIGS_h_GNUSTEP_RUBY_INCLUDE
#define __RIGS_h_GNUSTEP_RUBY_INCLUDE

// Mac OS X doesn't know about debug log.

#ifdef _MACOSX_

#define NSDebugLog NSLog
#define class_get_meta_class(class) objc_getMetaClass(class->name)
typedef struct objc_method_list * MethodList_t;

#endif

#ifdef GNUSTEP
#include <Foundation/NSDebug.h>
#endif

// VERSION is defined in GNUMakefile
// Defined in the Init function a static string with the RIGS version
#ifdef VERSION
#define RIGS_VERSION VERSION
#else
#define RIGS_VERSION ""
#endif


#endif
